@@ -1,87 +1,208 @@
 import datetime
 import json
+from copy import deepcopy
 from getpass import getpass
 from http.client import HTTPSConnection
 from .scraper import scrape_from_credentials, scrape_from_token, ExpiredSessionTokenException
 from .version import VERSION
 
 
-def check_for_json(obj, with_dates=False):
-    if type(obj) in (int, float, dict, list, str, bool) or obj is None:
-        return True
-    else:
-        return with_dates and obj in (datetime.date, datetime.datetime, datetime.time)
+class NuvolaOptions:
+    DATA_TYPES = {
+        "credentials": dict,
+        "verbose": bool,
+        "old_data": dict,
+        "connection": None,
+        "refresh_interval": datetime.timedelta,
+        "start_date": datetime.date,
+        "homeworks": {
+            "max_empty_days": int,
+            "backwards_refresh_date": datetime.timedelta
+        },
+        "timeWindows": {
+            "backwards_refresh_date": datetime.timedelta
+        },
+        "events": {
+            "backwards_refresh_date": datetime.timedelta
+        },
+        "topics": {
+            "max_empty_days": int,
+            "backwards_refresh_date": datetime.timedelta
+        }
+    }
+
+    def __init__(self, data=None):
+        if data:
+            self.data = data
+        else:
+            self.data = {
+                "credentials": None,
+                "old_data": {},
+                "verbose": False,
+                "refresh_interval": datetime.timedelta(hours=6),
+                "start_date": datetime.date(year=2020, month=8, day=30),
+                "homeworks": {
+                    "max_empty_days": 15 * 4,
+                    "backwards_refresh_date": datetime.timedelta(hours=15)
+                },
+                "topics": {
+                    "max_empty_days": 15 * 4,
+                    "backwards_refresh_date": datetime.timedelta(days=7)
+                }
+            }
+
+    def set(self, key, value):
+        if type(value) is self.DATA_TYPES[key] or type(value) is self.DATA_TYPES[key]:
+            self.data[key] = value
+        else:
+            raise TypeError(f"Invalid type: \"{self.DATA_TYPES[key]}\" required, \"{type(value)}\" provided")
+
+    def get(self, key):
+        try:
+            return self.data[key]
+        except KeyError:
+            raise self.KeyNotFoundException(f"The provided key (\"{key}\") can't be found.")
+
+    def keys(self):
+        return self.data.keys()
+
+    class KeyNotFoundException(Exception):
+        pass
 
 
 class Nuvola:
-    def __init__(self, id_student, **kwargs):
+    def __init__(self, id_student, options=NuvolaOptions(), old_data=None):
+        """
+        :param id_student: Student id, for now it can only be retrieved with a browser, sadly.
+        :param options: User defined options
+        :type id_student: int
+        :type options: NuvolaOptions
+        """
         self.id_student = id_student
+        self.options = options
         t1 = datetime.datetime.now()
-        self.conn = self.Connection()
-        print("[ OK ] Connection ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
-        if "old_data" in kwargs and type(kwargs["old_data"]) is dict:
-            self.__init_from_dict(kwargs["old_data"])
+        self.conn = self.Connection(self, self.options)
+        self.print("[ OK ] Connection ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
+        if type(old_data) is dict:
+            self.__init_from_dict(old_data)
             return
         t1 = datetime.datetime.now()
-        self.homeworks = self.Homeworks(self)
-        print("[ OK ] Homeworks ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
+        self.homeworks = self.Homeworks(self, self.options)
+        self.print("[ OK ] Homeworks ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
         t1 = datetime.datetime.now()
-        self.events = self.Events(self)
-        print("[ OK ] Events ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
+        self.events = self.Events(self, self.options)
+        self.print("[ OK ] Events ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
+        t1 = datetime.datetime.now()
+        self.topics = self.Topics(self, self.options)
+        self.print("[ OK ] Topics ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
         t1 = datetime.datetime.now()
         self.time_windows = self.__load_time_windows()
-        print("[ OK ] TimeWindows ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
+        self.print("[ OK ] TimeWindows ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
         self.active_time_window = self.__select_best_time_window()
 
+    def print(self, data):
+        """
+        Prints data only when verbose is active
+
+        :param data: Data to be print
+        """
+        if self.options.get("verbose"):
+            print(data)
+
     def __init_from_dict(self, obj):
+        """
+        :param obj: dict
+        """
+
         class FormatErrorException(Exception):
             pass
 
         class VersionMismatchException(Exception):
             pass
 
-        if not all([i in ("homeworks", "events", "timeWindows", "version") for i in obj.keys()]):
+        if not all([i in ("homeworks", "events", "timeWindows", "version", "topics") for i in obj.keys()]):
             raise FormatErrorException(obj)
         if obj["version"] == VERSION or input("Data from import obj is from another version, continue? (y,N) ") == "y":
             t1 = datetime.datetime.now()
-            self.homeworks = self.Homeworks(self, old_data=obj["homeworks"])
-            print("[ OK ] Homeworks ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
+            self.homeworks = self.Homeworks(self, self.options, obj["homeworks"])
+            self.print("[ OK ] Homeworks ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
             t1 = datetime.datetime.now()
-            self.events = self.Events(self, old_data=obj["events"])
-            print("[ OK ] Events ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
+            self.events = self.Events(self, self.options, obj["events"])
+            self.print("[ OK ] Events ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
+            t1 = datetime.datetime.now()
+            self.topics = self.Topics(self, self.options, obj["topics"])
+            self.print("[ OK ] Topics ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
             t1 = datetime.datetime.now()
             self.time_windows = self.__load_time_windows(obj["timeWindows"])
-            print("[ OK ] TimeWindows ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
+            self.print("[ OK ] TimeWindows ({} seconds)".format((datetime.datetime.now() - t1).total_seconds()))
             self.active_time_window = self.__select_best_time_window()
         else:
             raise VersionMismatchException(f"{VERSION} != {obj['version']}")
 
-    def get(self, area):
-        url = "/api-studente/v1/alunno/{}/{}".format(self.id_student, area)
+    def get(self, call):
+        """
+        Formats api request and sends it to /api-studente/v1/alunno/[call]
+
+        :param call: API call
+        :type call: str
+        :return: dict
+        """
+        url = "/api-studente/v1/alunno/{}/{}".format(self.id_student, call)
         d = self.conn.get_data(url)
         return d["valori"]
 
     def get_custom(self, custom_url):
+        """
+        Send raw call to connection.get_data without formatting to nuvola.madisoft.it/[custom_url].
+
+        :param custom_url: URL
+        :type custom_url: str
+        :return: dict
+        """
         d = self.conn.get_data(custom_url)
         return d
 
     def __load_time_windows(self, old_data=None):
-        if old_data is not None:
-            return [self.TimeWindow(self, i["raw"], old_data=i) for i in old_data]
-        return [self.TimeWindow(self, i) for i in self.get("frazioni-temporali")]
+        """
+        :param old_data:
+        :type old_data: dict
+        :rtype: list
+        """
+        if type(old_data) is list:
+            return [self.TimeWindow(self, i["raw"], self.options, i) for i in old_data]
+        return [self.TimeWindow(self, i, self.options) for i in self.get("frazioni-temporali")]
 
     def __load_irregularities(self):
+        """
+        :rtype: list
+        """
         return [self.Irregularity(i, self) for i in self.get("assenze")]
 
     def get_time_windows(self):
+        """
+        Get all available time windows
+        """
         for i in self.time_windows:
             yield i
 
+    def check_and_update_all(self, force=False):
+        h = [self.homeworks, self.events, self.topics, self.events]
+        for i in self.time_windows:
+            h.append(i)
+        for i in h:
+            i.check_and_update(force)
+
     class Connection:
         class RequestErrorException(Exception):
             pass
 
-        def __init__(self, **kwargs):
+        def __init__(self, parent, options):
+            """
+            :param parent: Parent nuvola object
+            :type parent: Nuvola
+            """
+            self.parent = parent
+            self.options = options
             self.c = HTTPSConnection("nuvola.madisoft.it")
             try:
                 with open("u.tok", "r") as f:
@@ -99,8 +220,13 @@ class Nuvola:
             try:
                 self.u_token = scrape_from_token(self.s_token)
             except ExpiredSessionTokenException:
-                print("Expired session token, please use credentials")
-                self.s_token, self.u_token = scrape_from_credentials(input("Username: "), getpass("Password: "))
+                if self.options.get("credentials") is not None:
+                    self.s_token, self.u_token = scrape_from_credentials(
+                        self.options.get("credentials")["username"], self.options.get("credentials")["password"])
+                else:
+                    self.parent.print("Expired session token, please use credentials")
+                    self.s_token, self.u_token = scrape_from_credentials(
+                        input("Username: "), getpass("Password: "))
             with open("s.tok", "w") as f:
                 f.write(self.s_token)
             with open("u.tok", "w") as f:
@@ -113,7 +239,7 @@ class Nuvola:
             if j == "Errore":
                 raise self.RequestErrorException(j)
             if "code" in j and j["code"] == 401:
-                print("Token expired, getting a new one...")
+                self.parent.print("Token expired, getting a new one...")
                 self.refresh_tokens()
                 return self.get_data(url)
             else:
@@ -125,12 +251,6 @@ class Nuvola:
         return "curl \"https://nuvola.madisoft.it/{}\" -H \"Authorization: {}\"".format(
             file.parent_class.ATTACHMENT_LINK.format(self.id_student, file.uuid), self.conn.u_token)
 
-    class IncompatibleTimeWindowException(Exception):
-        pass
-
-    class MissingSuitableTimeWindowException(Exception):
-        pass
-
     def __select_best_time_window(self):
         # Try to get entire year, else try to get current window
         for i in self.get_time_windows():
@@ -148,15 +268,20 @@ class Nuvola:
             raise self.IncompatibleTimeWindowException(tw)
 
     class Homeworks:
-        def __init__(self, parent, max_empty_days=15*4, **kwargs):
+        def __init__(self, parent, options, old_data=None):
+            """
+            :param parent: Parent nuvola object
+            :param options: Options object
+            :type parent: Nuvola
+            :type options: NuvolaOptions
+            """
             self.parent = parent
-            self.max_empty_days = max_empty_days
-            self.refresh_time = datetime.timedelta(hours=6)
+            self.options = options
             self.data = []
-            if "old_data" in kwargs and type(kwargs["old_data"]) is dict:
-                self.__init_from_dict(kwargs["old_data"])
+            if type(old_data) is dict:
+                self.__init_from_dict(old_data)
                 return
-            self.mod_time = datetime.datetime.now()
+            self.mod_time = datetime.datetime.fromtimestamp(0)
             self.load()
 
         def __init_from_dict(self, obj):
@@ -165,10 +290,17 @@ class Nuvola:
             self.mod_time = datetime.datetime.fromtimestamp(obj["mod_time"])
 
         def load(self):
-            date_s = datetime.date(year=2020, month=8, day=30) + datetime.timedelta(days=1)
-            date_e = date_s + datetime.timedelta(days=15)
             empty_count = 0
-            self.data = []
+            if self.data:
+                expired = list(self.get_by_expiration_date(
+                    datetime.date.today() - self.options.get("homeworks")["backwards_refresh_date"], 0, True))
+                for i in range(len(expired)):
+                    self.data.remove(expired[i])
+                date_s = datetime.date.today() + datetime.timedelta(days=1)
+            else:
+                date_s = self.options.get("start_date") + datetime.timedelta(days=1)
+
+            date_e = date_s + datetime.timedelta(days=15)
             while True:
                 # for each iteration we ask nuvola homeworks in a period of time of 15 days
                 c = self.parent.get("compito/elenco/{}/{}".format(
@@ -178,30 +310,37 @@ class Nuvola:
                 else:
                     self.data += [Nuvola.Homework(i) for i in c]
                     empty_count = 0
-                if empty_count >= self.max_empty_days / 15:
+                if empty_count >= self.options.get("homeworks")["max_empty_days"] / 15:
                     break
                 date_s = date_e + datetime.timedelta(days=1)
                 date_e += datetime.timedelta(days=15)
             self.mod_time = datetime.datetime.now()
 
         def check_and_update(self, force=False):
-            if force or datetime.datetime.now() > self.mod_time + self.refresh_time:
+            if force or datetime.datetime.now() > self.mod_time + self.options.get("refresh_interval"):
+                self.parent.print("Updating Homeworks...")
                 self.load()
 
-        def get_by_assignment_date(self, date):
+        def get_by_assignment_date(self, date, days=0):
+            """
+            :param date:
+            :param days:
+            :rtype: Nuvola.Homework
+            """
             self.check_and_update()
             if type(date) is not datetime.date:
                 raise TypeError(date)
             for i in self.data:
-                if i.date_assigned == date:
+                if date + datetime.timedelta(days=days) >= i.date_assigned >= date and i.date_assigned:
                     yield i
 
-        def get_by_expiration_date(self, date):
-            self.check_and_update()
+        def get_by_expiration_date(self, date, days=0, skip_check=False):
+            if not skip_check:
+                self.check_and_update()
             if type(date) is not datetime.date:
                 raise TypeError(date)
             for i in self.data:
-                if i.date_expired == date:
+                if date + datetime.timedelta(days=days) >= i.date_expired >= date and i.date_expired:
                     yield i
 
         def get_all(self):
@@ -209,11 +348,6 @@ class Nuvola:
             for i in self.data:
                 yield i
 
-        def set_refresh_time(self, time: datetime.timedelta):
-            if type(time) is not datetime.timedelta:
-                raise TypeError(time)
-            self.refresh_time = time
-
     class Homework:
         ATTACHMENT_LINK = "/api-studente/v1/alunno/{}/compito/allegato/{}"
 
@@ -229,14 +363,14 @@ class Nuvola:
             self.raw = h
 
     class Events:
-        def __init__(self, parent, **kwargs):
+        def __init__(self, parent, options, old_data=None):
             self.parent = parent
-            self.refresh_time = datetime.timedelta(hours=6)
+            self.options = options
             self.data = []
-            if "old_data" in kwargs and type(kwargs["old_data"]) is dict:
-                self.__init_from_dict(kwargs["old_data"])
+            if type(old_data) is dict:
+                self.__init_from_dict(old_data)
                 return
-            self.mod_time = datetime.datetime.now()
+            self.mod_time = datetime.datetime.fromtimestamp(0)
             self.load()
 
         def __init_from_dict(self, obj):
@@ -252,21 +386,17 @@ class Nuvola:
             self.mod_time = datetime.datetime.now()
 
         def check_and_update(self, force=False):
-            if force or datetime.datetime.now() > self.mod_time + self.refresh_time:
+            if force or datetime.datetime.now() > self.mod_time + self.options.get("refresh_interval"):
+                self.parent.print("Updating Events...")
                 self.load()
 
-        def set_refresh_time(self, time):
-            if type(time) is not datetime.timedelta:
-                raise TypeError(time)
-            self.refresh_time = time
-
         def get_all(self):
             self.check_and_update()
             for i in self.data:
                 yield i
 
         def get_by_date(self, date):
-            if type(date) is not date:
+            if type(date) is not datetime.date:
                 raise TypeError(date)
             self.check_and_update()
             for i in self.data:
@@ -294,7 +424,7 @@ class Nuvola:
         def get_by_id(self, id_):
             self.check_and_update()
             for i in self.data:
-                if not i.id_ == id_:
+                if i.id_ == id_:
                     return i
 
     class Event:
@@ -315,45 +445,42 @@ class Nuvola:
             self.border_color = e["coloreBordo"]
             self.id_notification = e["idNotifica"]
             self.date_start = datetime.datetime.fromisoformat(
-                e["dataInizio"].replace("00:00:00", e["oraInizio"]+":00"))
+                e["dataInizio"].replace("00:00:00", e["oraInizio"] + ":00"))
             self.date_end = datetime.datetime.fromisoformat(
                 e["dataFine"].replace("00:00:00", e["oraFine"] + ":00"))
             self.raw = e
 
     class TimeWindow:
-        def __init__(self, parent, w, **kwargs):
+        def __init__(self, parent, w, options, old_data=None):
             self.parent = parent
             self.id_ = w["id"]
             self.name = w["nome"]
             self.current = w["corrente"]
             self.raw = w
-            self.refresh_time = datetime.timedelta(hours=6)
-            if "old_data" in kwargs and type(kwargs["old_data"]) is dict:
-                self.__init_from_dict(kwargs["old_data"])
+            self.options = options
+            if type(old_data) is dict:
+                self.__init_from_dict(old_data)
                 return
-            self.mod_time = datetime.datetime.now()
+            self.mod_time = datetime.datetime.fromtimestamp(0)
             self.subjects = []
             self.load()
 
         def __init_from_dict(self, obj):
-            self.subjects = [self.Subject(self, i["raw"], old_data=i["marks"]) for i in obj["subjects"]]
+            self.subjects = [self.Subject(self, i["raw"], i["marks"]) for i in obj["subjects"]]
             self.mod_time = datetime.datetime.fromtimestamp(obj["mod_time"])
 
         def load(self):
+            self.subjects = []
             s = self.parent.get("frazione-temporale/{}/voti/materie".format(self.id_))
             for i in s:
                 self.subjects.append(self.Subject(self, i))
             self.mod_time = datetime.datetime.now()
 
         def check_and_update(self, force=False):
-            if force or datetime.datetime.now() > self.mod_time + self.refresh_time:
+            if force or datetime.datetime.now() > self.mod_time + self.options.get("refresh_interval"):
+                self.parent.print("Updating TimeWindow...")
                 self.load()
 
-        def set_refresh_time(self, time: datetime.timedelta):
-            if type(time) is not datetime.timedelta:
-                raise TypeError(time)
-            self.refresh_time = time
-
         def get_subject_by_name(self, name):
             self.check_and_update()
             for i in self.subjects:
@@ -372,26 +499,27 @@ class Nuvola:
                     return id_
 
         class Subject:
-            def __init__(self, parent, s, **kwargs):
+            def __init__(self, parent, s, old_data=None):
                 self.parent = parent
                 self.id_ = s["id"]
                 self.name = s["materia"]
                 self.type = s["tipo"]
                 self.raw = s
-                if "old_data" in kwargs and type(kwargs["old_data"]) is list:
-                    self.__init_from_dict(kwargs["old_data"])
+                if type(old_data) is list:
+                    self.__init_from_dict(old_data)
                     return
                 self.marks = []
                 self.load()
 
             def __init_from_dict(self, obj):
-                self.marks = [self.Mark(i) for i in obj]
+                self.marks = [self.Mark(i, self) for i in obj]
 
             def load(self):
+                self.marks = []
                 m = self.parent.parent.get(
                     "frazione-temporale/{}/voti/materia/{}".format(self.parent.id_, self.id_))
                 for i in m[0]["voti"]:
-                    self.marks.append(self.Mark(i))
+                    self.marks.append(self.Mark(i, self))
 
             def get_all(self):
                 self.parent.check_and_update()
@@ -431,7 +559,10 @@ class Nuvola:
                         yield i
 
             class Mark:
-                def __init__(self, m):
+                def __init__(self, m, parent):
+                    self.parent = parent
+                    self.subject = self.parent.name
+                    self.subject_id = self.parent.id_
                     self.date = datetime.datetime.fromisoformat(m["data"]).date()
                     self.teacher = m["docente"]
                     self.type_ = m["tipologia"]
@@ -477,10 +608,131 @@ class Nuvola:
         def __get_details(self):
             return self.parent.get_custom("/api-studente/v1/assenza/" + str(self.id))["dettaglio"]
 
+    class Topics:
+        def __init__(self, parent, options, old_data=None):
+            self.parent = parent
+            self.options = options
+            self.data = []
+            if type(old_data) is dict:
+                self.__init_from_dict(old_data)
+                return
+            self.mod_time = datetime.datetime.fromtimestamp(0)
+            self.load()
+
+        def __init_from_dict(self, obj):
+            for i in obj["data"]:
+                self.data.append(Nuvola.Topic(i["lesson"], i["lesson"]["argomenti"], i["class"], i["class_id"]))
+            self.mod_time = datetime.datetime.fromtimestamp(obj["mod_time"])
+
+        def load(self):
+            if self.data:
+                expired = list(self.get_by_date(
+                    datetime.date.today() - self.options.get("topics")["backwards_refresh_date"], True))
+                for i in range(len(expired)):
+                    self.data.remove(expired[i])
+                date_s = datetime.date.today() + datetime.timedelta(days=1)
+            else:
+                date_s = self.options.get("start_date") + datetime.timedelta(days=1)
+            date_e = date_s + datetime.timedelta(days=15)
+            empty_count = 0
+
+            while True:
+                # for each iteration we ask nuvola homeworks in a period of time of 15 days
+                c = self.parent.get("argomento-lezione/elenco/{}/{}".format(
+                    date_s.strftime("%d-%m-%Y"), date_e.strftime("%d-%m-%Y")))
+                emp = True
+                for i in c:
+                    for j in i["ore"]:
+                        if j["argomenti"]:
+                            emp = False
+                            if len(j["argomenti"]) == 1:
+                                self.data.append(Nuvola.Topic(j, j["argomenti"][0], i["classe"], i["classeId"]))
+                            else:
+                                for k in range(len(j["argomenti"])):
+                                    self.data.append(Nuvola.Topic(j, j["argomenti"][k], i["classe"], i["classeId"]))
+                if emp:
+                    empty_count += 1
+                if empty_count >= self.options.get("topics")["max_empty_days"] / 15:
+                    break
+                date_s = date_e + datetime.timedelta(days=1)
+                date_e += datetime.timedelta(days=15)
+            self.mod_time = datetime.datetime.now()
+
+        def check_and_update(self, force=False):
+            if force or datetime.datetime.now() > self.mod_time + self.options.get("refresh_interval"):
+                self.parent.print("Updating Topics...")
+                self.load()
+
+        def get_all(self):
+            self.check_and_update()
+            for i in self.data:
+                yield i
+
+        def get_by_date(self, date, skip_check):
+            if type(date) is not datetime.date:
+                raise TypeError(date)
+            if not skip_check:
+                self.check_and_update()
+            for i in self.data:
+                if i.date == date:
+                    yield i
+
+        def get_by_teacher(self, teacher):
+            self.check_and_update()
+            for i in self.data:
+                if i.teacher == teacher:
+                    yield i
+
+        def get_by_subject(self, subject, search=False):
+            self.check_and_update()
+            for i in self.data:
+                if i.subject == subject or search and subject in i.subject:
+                    yield i
+
+        def get_by_type(self, type_):
+            self.check_and_update()
+            for i in self.data:
+                if not i.type == type_:
+                    yield i
+
+        def get_by_id(self, id_):
+            self.check_and_update()
+            for i in self.data:
+                if i.id_ == id_:
+                    return i
+
+    class Topic:
+        def __init__(self, t, a, class_, class_id):
+            self.class_ = class_
+            self.class_id = class_id
+
+            self.lesson = t["numeroOra"]
+            self.time_start = datetime.datetime.fromisoformat(
+                t["giorno"].replace("00:00:00", t["inizioOra"] + ":00")).time()
+            self.time_end = datetime.datetime.fromisoformat(
+                t["giorno"].replace("00:00:00", t["fineOra"] + ":00")).time()
+            self.date = datetime.datetime.fromisoformat(t["giorno"]).date()
+
+            if a["cofirme"] or a["compresenza"]:
+                raise
+            self.id_ = a["id"]
+            self.type = a["tipo"]
+            self.subject = a["materia"]
+            self.name = a["nomeArgomento"]
+            self.long_description = a["descrizioneEstesa"]
+            self.co_presence = a["compresenza"]
+            self.teacher = a["docente"]
+            self.notes = a["annotazioni"]
+            self.attachments = [Nuvola.File(i, self.__class__) for i in a["allegati"]]
+            self.youtube_link = a["video_youtube"]
+            t_r = deepcopy(t)
+            t_r["argomenti"] = deepcopy(a)
+            self.raw = t_r
+
     class File:
-        def __init__(self, f, parent, **kwargs):
-            if "old_data" in kwargs and type(kwargs["old_data"]) is dict:
-                self.__init_from_dict(kwargs["old_data"])
+        def __init__(self, f, parent, old_data=None):
+            if type(old_data) is dict:
+                self.__init_from_dict(old_data)
                 return
             self.parent = parent
             self.id_ = f["id"]
@@ -496,6 +748,7 @@ class Nuvola:
         self.events.check_and_update(update_first)
         for i in self.get_time_windows():
             i.check_and_update(update_first)
+        self.topics.check_and_update(update_first)
 
         output = {
             "homeworks": {
@@ -506,6 +759,10 @@ class Nuvola:
                 "mod_time": self.events.mod_time.timestamp(),
                 "data": []
             },
+            "topics": {
+                "mod_time": self.topics.mod_time.timestamp(),
+                "data": []
+            },
             "timeWindows": [],
             "version": VERSION
         }
@@ -534,4 +791,18 @@ class Nuvola:
                     t_s["marks"].append(m.raw)
                 t_tw["subjects"].append(t_s)
             output["timeWindows"].append(t_tw)
+
+        # topics
+        for t in self.topics.get_all():
+            output["topics"]["data"].append({
+                "lesson": t.raw,
+                "class": t.class_,
+                "class_id": t.class_id
+            })
         return output
+
+    class IncompatibleTimeWindowException(Exception):
+        pass
+
+    class MissingSuitableTimeWindowException(Exception):
+        pass
