@@ -1,6 +1,7 @@
 import seleniumwire.webdriver
-from requests import get
+import requests
 from json.decoder import JSONDecodeError
+from simplejson.decoder import JSONDecodeError as JSONDecodeError_
 
 opt = seleniumwire.webdriver.ChromeOptions()
 opt.headless = True
@@ -14,11 +15,15 @@ class ExpiredSessionTokenException(Exception):
     pass
 
 
+class GenericErrorException(Exception):
+    pass
+
+
 def scrape_from_token(session_token):
     try:
-        return get("https://nuvola.madisoft.it/api-studente/v1/login-from-web",
+        return requests.get("https://nuvola.madisoft.it/api-studente/v1/login-from-web",
                    cookies={"nuvola": str(session_token)}).json()["token"]
-    except JSONDecodeError:
+    except (JSONDecodeError, JSONDecodeError_):
         raise ExpiredSessionTokenException
 
 
@@ -34,7 +39,8 @@ def scrape_from_credentials(user, pwd):
     d.close()
 
     try:
-        return session_token, get("https://nuvola.madisoft.it/api-studente/v1/login-from-web",
-                                  cookies={"nuvola": str(session_token)}).json()["token"]
+        r = requests.get("https://nuvola.madisoft.it/api-studente/v1/login-from-web",
+                         cookies={"nuvola": str(session_token)})
+        return session_token, r.json()["token"]
     except JSONDecodeError:
-        raise InvalidCredentialsException
+        raise InvalidCredentialsException(r.text)
